/*

    GDS7 v0.1 interactive digital image transfer software for DS-7 camera
    Copyright (C) 1998 Matthew G. Martin

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


*/
#include "gds7.h"
#include "ds7lib.h"
#include "ops.h"
#include "ui.h"
#include "blank.xpm"
#include "mm.xpm"
#include "callbacks.h"

GtkWidget *statframe,*pixcount, *camid, *picwin,*align,*toolbar;
GtkWidget *pictable=NULL;
GtkWidget *window,*blankmap;
GtkWidget *progress,*haltbutton;
GtkWidget *mainbar,*filemenu,*popmenu;
GtkWidget *prev_sel,*dev_sel,*size_sel,*gam_sel,*col_sel;
GtkAcceleratorTable *main_accel;

/**********************************************************************
  Configuration Dialog
 **********************************************************************/

/* Allow user to configure the various options
*/
#define ConfigMenuItem(x)   tbox=gtk_hbox_new(FALSE, 0);\
  temp=gtk_alignment_new(0,0.5,0,0);\
  temp2=gtk_label_new(x);\
  gtk_container_add(GTK_CONTAINER(temp),temp2);\
  gtk_box_pack_start (GTK_BOX (tbox), temp,TRUE, TRUE, 5);\
  gtk_widget_show(temp);\
  gtk_widget_show(temp2);

#define BaudMenuItem(x) \
  temp=gtk_menu_item_new();\
  temp2=gtk_label_new(x);\
  gtk_widget_show(temp2);\
  gtk_container_add(GTK_CONTAINER(temp),temp2);\
  gtk_signal_connect_object(GTK_OBJECT(temp), "activate",\
			    GTK_SIGNAL_FUNC(baudset) ,GTK_OBJECT(temp2));\
  gtk_menu_append(GTK_MENU(baudmenu),temp);\
  if (baudrate<=atoi(x))\
     gtk_menu_set_active(GTK_MENU(baudmenu),item_cntr);\
  item_cntr++;   \
  gtk_widget_show(temp);\

void config_dialog(){
  GtkWidget *dialog,*temp,*temp2,*tbox,*baudmenu;
  GtkAdjustment *adj;
  int item_cntr;

  dialog=gtk_dialog_new();
  temp=gtk_label_new("Configuration");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), temp,
                    TRUE, TRUE, 0);
  gtk_widget_show(temp);

  /* Previewer Config */
  ConfigMenuItem("Previewer");

  prev_sel=gtk_entry_new();
  gtk_entry_set_text(GTK_ENTRY(prev_sel),preview_cmd);
  gtk_box_pack_start (GTK_BOX (tbox), prev_sel,TRUE, TRUE, 0);
  gtk_widget_show(prev_sel);

  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), tbox,TRUE, TRUE, 0);
  gtk_widget_show(tbox);

  /* Device Config */

  ConfigMenuItem("Cam Port Device");

   dev_sel=gtk_entry_new();
   gtk_entry_set_text(GTK_ENTRY(dev_sel),camdev);
   gtk_box_pack_start (GTK_BOX (tbox), dev_sel,TRUE, TRUE, 0);
   gtk_widget_show(dev_sel);

  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), tbox,TRUE, TRUE, 0);
  gtk_widget_show(tbox);

  /* Baud Rate */
  baudmenu=gtk_menu_new();
  item_cntr=0;
  menubaud=baudrate;

  BaudMenuItem("57600");
  BaudMenuItem("38400");
  BaudMenuItem("19200");
  BaudMenuItem("9600");

  ConfigMenuItem("Baud Rate");

  temp=gtk_option_menu_new();
  gtk_option_menu_set_menu(GTK_OPTION_MENU(temp),baudmenu);

  gtk_box_pack_start (GTK_BOX (tbox), temp,TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), tbox,TRUE, TRUE, 0);
  gtk_widget_show(temp);
  gtk_widget_show(tbox);


  /* Thumbnail Size */

  ConfigMenuItem("Thumb Size ");

  adj= (GtkAdjustment *) gtk_adjustment_new (thumbscale, 0.5, 5.0, 0.5,1,1);

   size_sel=gtk_spin_button_new(adj,0.2,1);
   gtk_box_pack_start (GTK_BOX (tbox), size_sel,TRUE, TRUE, 0);
   gtk_widget_show(size_sel);

  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), tbox,TRUE, TRUE, 0);
  gtk_widget_show(tbox);


  /* Thumbnail Gamma */

  ConfigMenuItem("Thumb Gamma ");

  adj= (GtkAdjustment *) gtk_adjustment_new (thumbgam, 0.2, 5.0, 0.2,1,1);

   gam_sel=gtk_spin_button_new(adj,0.2,1);
   gtk_box_pack_start (GTK_BOX (tbox), gam_sel,TRUE, TRUE, 0);
   gtk_widget_show(gam_sel);

  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), tbox,TRUE, TRUE, 0);
  gtk_widget_show(tbox);

  /* Columns */

  ConfigMenuItem("Columns");

  adj= (GtkAdjustment *) gtk_adjustment_new (cols, 1, 20.0, 1,1,1);

   col_sel=gtk_spin_button_new(adj,1,1);
   gtk_box_pack_start (GTK_BOX (tbox), col_sel,TRUE, TRUE, 0);
   gtk_widget_show(col_sel);

  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), tbox,TRUE, TRUE, 0);
  gtk_widget_show(tbox);


  /* Finish buttons */

  temp=gtk_button_new_with_label("OK");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), temp,
                    TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (temp), "clicked",
		      GTK_SIGNAL_FUNC(config_update), GTK_OBJECT(dialog));
  gtk_signal_connect (GTK_OBJECT (temp), "clicked",
		      GTK_SIGNAL_FUNC(killdialog), GTK_OBJECT(dialog));
  gtk_widget_show(temp);

  temp=gtk_button_new_with_label("Cancel");
  gtk_signal_connect (GTK_OBJECT (temp), "clicked",
		      GTK_SIGNAL_FUNC(killdialog), GTK_OBJECT(dialog));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), temp,
                    TRUE, TRUE, 0);

  gtk_widget_show(temp);
  gtk_widget_show(dialog);
};


/***********************************************************************
  Create menu
 ***********************************************************************/

void make_menu(){
  GtkWidget *temp;

  filemenu=gtk_menu_new();

  temp=gtk_menu_item_new_with_label("Save\nSelected");
  gtk_signal_connect_object(GTK_OBJECT(temp), "activate", GTK_SIGNAL_FUNC(allsave) ,NULL);
  gtk_menu_append(GTK_MENU(filemenu), temp);
  gtk_widget_show(temp);

  temp=gtk_menu_item_new_with_label("Quit");
  gtk_signal_connect_object(GTK_OBJECT(temp), "activate", gtk_main_quit,NULL);
  gtk_menu_append(GTK_MENU(filemenu), temp);
  gtk_widget_show(temp);

  /* The Main Menubar */
  mainbar=gtk_menu_bar_new();
  //  gtk_widget_show(mainbar);
  //  temp=gtk_menu_item_new_with_label("File");
  //  gtk_widget_show(temp);
  //  gtk_menu_item_set_submenu(GTK_MENU_ITEM(temp),filemenu);
  //  gtk_menu_bar_append(GTK_MENU_BAR(mainbar),temp);



  popmenu=gtk_menu_new();
  gtk_menu_set_accelerator_table(GTK_MENU(popmenu),main_accel);

  temp=gtk_menu_item_new_with_label("Save");
  gtk_signal_connect_object(GTK_OBJECT(temp), "activate",
			    GTK_SIGNAL_FUNC(pop_get),NULL);
  gtk_menu_append(GTK_MENU(popmenu), temp);
  gtk_widget_install_accelerator(GTK_WIDGET(temp),main_accel,"activate",'S',0);
  gtk_widget_show(temp);

  temp=gtk_menu_item_new_with_label("Save As...");
  gtk_signal_connect_object(GTK_OBJECT(temp), "activate",
			    GTK_SIGNAL_FUNC(saveas),NULL);
  gtk_menu_append(GTK_MENU(popmenu), temp);
  gtk_widget_install_accelerator(GTK_WIDGET(temp),main_accel,"activate",'A',0);
  gtk_widget_show(temp);

  temp=gtk_menu_item_new_with_label("Preview <p>");
  gtk_signal_connect_object(GTK_OBJECT(temp), "activate",
			    GTK_SIGNAL_FUNC(preview),NULL);
  gtk_menu_append(GTK_MENU(popmenu), temp);
  gtk_widget_install_accelerator(GTK_WIDGET(temp),main_accel,"activate",'P',0);
  gtk_widget_show(temp);

  temp=gtk_menu_item_new_with_label("Thumb <t>");
  gtk_signal_connect_object(GTK_OBJECT(temp), "activate",
			    GTK_SIGNAL_FUNC(pop_thumb),NULL);
  gtk_menu_append(GTK_MENU(popmenu), temp);
  gtk_widget_install_accelerator(GTK_WIDGET(temp),main_accel,"activate",'T',0);
  gtk_widget_show(temp);

  temp=gtk_menu_item_new_with_label("Info");
  gtk_signal_connect_object(GTK_OBJECT(temp), "activate",
			    GTK_SIGNAL_FUNC(thumb_get_info),NULL);
  gtk_menu_append(GTK_MENU(popmenu), temp);
  gtk_widget_install_accelerator(GTK_WIDGET(temp),main_accel,"activate",'I',0);
  gtk_widget_show(temp);

};


/* 
   Update display 
*/
void gtk_sync(){
      while (gtk_events_pending())
        gtk_main_iteration();
};


/* 
Make pixel number, shouldn't this be in a lib somewhere? 
*/
void pixelcalc(GdkColor *color){
  color->pixel=(gulong)(
			color->red*(255/65535)*65536+
			color->green*256*(255/65535)+
			color->blue*(255/65535)
			);
};


/* Brings up a message for the user */

void popup_dialog(char *mesg){
  GtkWidget *dialog,*temp;

  if (mesg==NULL) return;

  dialog=gtk_dialog_new();
  gtk_container_border_width(GTK_CONTAINER (GTK_DIALOG (dialog)->vbox),5);
  temp=gtk_label_new(mesg);
  gtk_widget_set_name(temp,"alert_text");\
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), temp,
                    TRUE, TRUE, 0);
  gtk_widget_show(temp);
  temp=gtk_button_new_with_label("OK");
  gtk_widget_set_name(temp,"alert_button");
  gtk_signal_connect (GTK_OBJECT (temp), "clicked",
		      GTK_SIGNAL_FUNC(killdialog), GTK_OBJECT(dialog));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), temp,
                    TRUE, TRUE, 0);

  gtk_widget_show(temp);
  gtk_widget_show(dialog);
};

/* Some macros to make this cheezy help window approach easier */

#define HELP_ADD(x) \
  gtk_box_pack_start (GTK_BOX (mainbox), x,FALSE,FALSE, 0);\
  gtk_widget_show(x);

#define HELP_TEXT(x) \
  temp=gtk_label_new(x);\
  gtk_label_set_justify(GTK_LABEL(temp),GTK_JUSTIFY_FILL);\
  HELP_ADD(temp);

#define HELP_CENTER_ADD(x) \
  frame=gtk_alignment_new(0.5,0.5,0,0);\
  gtk_container_add(GTK_CONTAINER(frame),x);\
  gtk_widget_show(x);\
  HELP_ADD(frame);

#define HELP_SUB_ADD(x) \
  temp=gtk_label_new(x);\
  gtk_widget_set_name(temp,"help_sub_title");\
  gtk_misc_set_alignment(GTK_MISC(temp),0,0.5);\
  gtk_box_pack_start (GTK_BOX (mainbox), temp,TRUE, TRUE, 10);\
  gtk_widget_show(temp);


#define TABLE_BUTTON_ADD(button,desc,accel) \
  temp=gtk_toggle_button_new_with_label(button);\
  gtk_table_attach(GTK_TABLE(table),temp,0,1,row,row+1,GTK_FILL,0,0,0);\
  gtk_widget_show(temp);\
  temp=gtk_label_new(desc);\
  gtk_misc_set_alignment(GTK_MISC(temp),0,0.5);\
  gtk_table_attach(GTK_TABLE(table),temp,1,2,row,row+1,GTK_FILL,0,5,0);\
  gtk_widget_show(temp);\
  temp=gtk_label_new(accel);\
  gtk_widget_show(temp);\
  gtk_table_attach(GTK_TABLE(table),temp,2,3,row,row+1,GTK_FILL,0,5,0);\
  row++;

#define TABLE_ADD(term,desc,extra,accel) \
  temp=gtk_label_new(term);\
  gtk_widget_show(temp);\
  gtk_table_attach(GTK_TABLE(table),temp,0,1,row,row+1,GTK_FILL,0,5,0);\
  temp=gtk_label_new(desc);\
  gtk_misc_set_alignment(GTK_MISC(temp),0,0.5);\
  gtk_widget_show(temp);\
  gtk_table_attach(GTK_TABLE(table),temp,1,2,row,row+1,GTK_FILL,0,5,0);\
  temp=gtk_label_new(extra);\
  gtk_misc_set_alignment(GTK_MISC(temp),0,0.5);\
  gtk_widget_show(temp);\
  gtk_table_attach(GTK_TABLE(table),temp,2,3,row,row+1,GTK_FILL,0,5,0);\
  temp=gtk_label_new(accel);\
  gtk_widget_show(temp);\
  gtk_table_attach(GTK_TABLE(table),temp,3,4,row,row+1,GTK_FILL,0,5,0);\
  row++;

/* Brings up a message for the user */

void help_dialog(){
  GtkWidget *dialog,*mainbox,*temp,*temp2,*frame,*table;
  GdkImlibImage *im;
  GdkPixmap *p;
  GdkBitmap *m;
  int row;

  dialog=gtk_dialog_new();
  temp=gtk_scrolled_window_new(NULL,NULL);
  gtk_widget_set_usize(temp,600,300);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(temp)
				   ,GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), temp,
                    TRUE, TRUE, 0);
  gtk_widget_show(temp);

  mainbox=gtk_vbox_new(FALSE,0);
  gtk_container_add(GTK_CONTAINER(temp),mainbox);

  temp=gtk_label_new("GDS7");
  gtk_widget_set_name(temp,"help_title");
  HELP_ADD(temp);

  temp=gtk_label_new("Interactive Image Transfer for the \
Fuji DS7 Digital Camera");
  HELP_ADD(temp);

  temp=gtk_hseparator_new();
  HELP_ADD(temp);

  HELP_SUB_ADD("Overview");

  HELP_TEXT("Welcome to GDS7, Version 0.1.

You can use this program to browse pictures from your DS7 interractively
or select a batch for download.

When the program first runs, it will try to initialize the camera.  If
successful, a set of thumbnails will appear in the window.  Each
corresponds to an image stored within the camera.  Because of the time
involved, none of the thumbnail images are loaded by default.

If no thumbnails appear, click on the \"Configure\" button and try
changing the port and/or baud-rate settings; the defaults probably do
not match your setup.

Once you have a screen of 'nails, the help sections below should
give you an idea of what to do next.

Note that if you are like me and like to use the keyboard, all of the
buttons are accelerated and you can step through the thumbnails and
filenames with the tab and/or arrow keys. Use <Space> or <Enter> to
toggle the thumbnails.");


HELP_SUB_ADD("Batch Download");

  HELP_TEXT("Select pictures by \"pressing in\" the thumbnail,
the text window will tint red. Once the picture is saved,
 the text window will tint green and will be ignored in 
subsequent save requests (unless the filename is changed).
");

  table=gtk_table_new(2,2,FALSE);

  temp=gtk_label_new("Unsaved");
  gtk_widget_show(temp);
  gtk_table_attach(GTK_TABLE(table),temp,0,1,0,1,GTK_FILL,0,0,0);

  temp2=gtk_entry_new();
  gtk_widget_set_name(temp2,"selected_text");
  gtk_entry_set_text(GTK_ENTRY(temp2),"myfile.jpg");
  gtk_table_attach(GTK_TABLE(table),temp2,0,1,1,2,GTK_FILL,0,0,0);
  gtk_widget_show(temp2);

  temp=gtk_label_new("Saved");
  gtk_widget_show(temp);
  gtk_table_attach(GTK_TABLE(table),temp,1,2,0,1,GTK_FILL,0,0,0);

  temp2=gtk_entry_new();
  gtk_widget_set_name(temp2,"saved_text");
  gtk_entry_set_text(GTK_ENTRY(temp2),"myfile.jpg");
  gtk_table_attach(GTK_TABLE(table),temp2,1,2,1,2,GTK_FILL,0,0,0);
  gtk_widget_show(temp2);
  row++;

  HELP_CENTER_ADD(table);

  temp=gtk_label_new("
Batch commands are on the toolbar");
  HELP_ADD(temp);

  /***********************************************************************
     Menubar help
  **********************************************************************/
  HELP_SUB_ADD("Toolbar");

  table=gtk_table_new(5,2,FALSE);
  row=0;

  temp=gtk_label_new("Toolbar buttons are global");
  gtk_widget_show(temp);
  gtk_table_attach(GTK_TABLE(table),temp,0,2,row,row+1,GTK_FILL,0,0,0);
  row++;
  TABLE_ADD("Button","Action","Accel","");
  temp=gtk_hseparator_new();
  gtk_widget_show(temp);
  gtk_table_attach(GTK_TABLE(table),temp,0,4,row,row+1,GTK_FILL,0,0,0);
  row++;
  TABLE_BUTTON_ADD("Init","Connects with camera","ctrl-I");
  TABLE_BUTTON_ADD("Thumbs","Download Thumbnails for Selected Pix","ctrl-t");
  TABLE_BUTTON_ADD("Save","Save all selected pictures","ctrl-s");
  TABLE_BUTTON_ADD("Deselect","Deselect all thumbnails","ctrl-d");
  TABLE_BUTTON_ADD("Help","View this help window","ctrl-h");

  HELP_CENTER_ADD(table);


  HELP_SUB_ADD("Over Thumbnail");

  HELP_TEXT("Only the clicked-upon image is affected");

  table=gtk_table_new(5,4,FALSE);
  row=0;

  TABLE_ADD("Action","","Result","Accel Key");
  
  temp=gtk_hseparator_new();
  gtk_widget_show(temp);
  gtk_table_attach(GTK_TABLE(table),temp,0,4,row,row+1,GTK_FILL,0,0,0);
  row++;
  TABLE_ADD("Double Click","","Loads Thumbnail","T");
  TABLE_ADD("Right Button","Popup Menu with","","");
  TABLE_ADD("","Save:","saves picture","S");
  TABLE_ADD("","Save as:","prompts for filename and saves","A");
  TABLE_ADD("","Preview:","loads image into a display application","P");
  TABLE_ADD("","Thumb:","loads the thumbnail","T");
  TABLE_ADD("","Info:","currently unimplemented","I");
  HELP_CENTER_ADD(table);

  HELP_SUB_ADD("Configure");
  HELP_TEXT("\
Allows configuration of:
  Camera serial port and transfer rate.
       If errors are frequent or data is corrupted, lower the transfer rate.
                    (better yet, help improve the transfer code)
  Thumbnail size and gamma correction (no effect on downloaded image)
  Preview program
  Number of thumbnails in each row.

Note: Configure currently destroys the thumbnail data.");

  temp=gtk_hseparator_new();
  HELP_ADD(temp);

  im=gdk_imlib_create_image_from_xpm_data(mm_xpm);
  gdk_imlib_render(im,im->rgb_width,im->rgb_height);
  p=gdk_imlib_move_image(im);
  m=gdk_imlib_move_mask(im);
  temp=gtk_pixmap_new(p,m);
  HELP_ADD(temp);

  temp=gtk_label_new("Written by Matt Martin (matt.martin@ieee.org)");
  HELP_ADD(temp);

  gtk_widget_show(mainbox);

  temp=gtk_button_new_with_label("OK");
  gtk_signal_connect (GTK_OBJECT (temp), "clicked",
		      GTK_SIGNAL_FUNC(killdialog), GTK_OBJECT(dialog));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), temp,
                    TRUE, TRUE, 0);

  gtk_widget_show(temp);

  gtk_widget_show(dialog);
  gdk_imlib_free_pixmap(p);
  gdk_imlib_kill_image(im);
};


/* Create Main Window */

void main_win(){
  
    /* GtkWidget is the storage type for widgets */
    GtkWidget *box1;
    GtkWidget *statusbox,*temp;

    /* create a new window */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

    main_accel=gtk_accelerator_table_new();
    gtk_window_add_accelerator_table(GTK_WINDOW(window),main_accel);

    gtk_window_set_title (GTK_WINDOW (window), "DS-7 Image Transfer");

    /* Here we just set a handler for delete_event that immediately
     * exits GTK. */
    gtk_signal_connect (GTK_OBJECT (window), "delete_event",
                        GTK_SIGNAL_FUNC (delete_event), NULL);

    make_menu();
    
    /* sets the border width of the window. */
    gtk_container_border_width (GTK_CONTAINER (window), 3);

    /* we create a box to pack widgets into.  this is described in detail
     * in the "packing" section below.  The box is not really visible, it
     * is jus tused as a tool to arrange widgets. */
    box1 = gtk_vbox_new(FALSE, 0);

    /* the scrollable window for thumbnails */

    picwin=gtk_scrolled_window_new(NULL,NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(picwin)
				   ,GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
    gtk_widget_set_usize(picwin,600,300);

    align=gtk_alignment_new(0,0,0,0);
    gtk_container_add(GTK_CONTAINER(picwin),align);
    gtk_widget_show(align);

    gtk_widget_show(picwin);


    /* Do the Toolbar Thang */
    
    toolbar=gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL,GTK_TOOLBAR_BOTH);
    temp=gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),"Init",
				 "Initialize (ctrl-I)",
			    "useless",NULL,GTK_SIGNAL_FUNC(cam_init),
			    (gpointer)"");
    gtk_widget_install_accelerator(GTK_WIDGET(temp),main_accel,"clicked",\
				   'I',GDK_CONTROL_MASK);

    temp=gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),"Thumbs",
			    "Get All Selected Thumbnails ctrl-T","waste",
			    NULL,GTK_SIGNAL_FUNC(getthumbs),(gpointer)"");
    gtk_widget_install_accelerator(GTK_WIDGET(temp),main_accel,"clicked",\
				   'T',GDK_CONTROL_MASK);

    temp=gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),"Save",
			    "Download and Save all selected (ctrl-S)","save"
			    ,NULL,GTK_SIGNAL_FUNC(allsave),(gpointer)"");
    gtk_widget_install_accelerator(GTK_WIDGET(temp),main_accel,"clicked",\
				   'S',GDK_CONTROL_MASK);

    temp=gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),"Deselect",
			    "Deselect all pictures (ctrl-D)","deselect"
			    ,NULL,GTK_SIGNAL_FUNC(deselect),(gpointer)"");
    gtk_widget_install_accelerator(GTK_WIDGET(temp),main_accel,"clicked",\
				   'D',GDK_CONTROL_MASK);

    temp=gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),"Help",
			    "Show Help (ctrl-H)","help"
			    ,NULL,GTK_SIGNAL_FUNC(show_help),(gpointer)"");
    gtk_widget_install_accelerator(GTK_WIDGET(temp),main_accel,"clicked",\
				   'H',GDK_CONTROL_MASK);

    temp=gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),"Configure",
			    "Settings (ctrl-C)","conf"
			    ,NULL,GTK_SIGNAL_FUNC(config_dialog),(gpointer)"");
    gtk_widget_install_accelerator(GTK_WIDGET(temp),main_accel,"clicked",\
				   'C',GDK_CONTROL_MASK);

    gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
    gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
    gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
    gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
    temp=gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),"Quit","Exit ctrl-q",
			    "quitter",NULL,GTK_SIGNAL_FUNC(gtk_main_quit),
			    (gpointer)"");
    gtk_widget_install_accelerator(GTK_WIDGET(temp),main_accel,"clicked",\
				   'Q',GDK_CONTROL_MASK);


    /* Setup the status indicators */
    statusbox = gtk_hbox_new(FALSE, 0);

    /* Camera ID Indicator */
    camid= gtk_label_new("Camera:");
    gtk_box_pack_start(GTK_BOX(statusbox),camid,FALSE,FALSE,0);
    gtk_widget_show(camid);

    camid = gtk_statusbar_new();
    gtk_statusbar_push(GTK_STATUSBAR(camid),1,"No-Cam");
    gtk_widget_show(camid);

    gtk_box_pack_start(GTK_BOX(statusbox), camid, FALSE,FALSE, 0);
    gtk_container_border_width(GTK_CONTAINER(statusbox),3);

    statframe=gtk_frame_new("");

    /* Picture Count */
    pixcount= gtk_label_new("Pictures:");
    gtk_widget_show(pixcount);
    gtk_box_pack_start(GTK_BOX(statusbox),pixcount,FALSE,FALSE,0);

    pixcount = gtk_statusbar_new();
    gtk_widget_show(pixcount);
    gtk_statusbar_push(GTK_STATUSBAR(pixcount),1,"X");
    gtk_box_pack_start(GTK_BOX(statusbox), pixcount, FALSE, FALSE, 0);

    /* Transfer Progress Bar */

    progress=gtk_progress_bar_new();
    gtk_widget_set_name(progress,"progress_bar");
    gtk_box_pack_start(GTK_BOX(statusbox),progress,TRUE,TRUE,10);
    gtk_widget_show(progress);

    haltbutton=gtk_toggle_button_new_with_label("Cancel");
    gtk_widget_set_name (haltbutton, "cancel_button");
    gtk_box_pack_start(GTK_BOX(statusbox),haltbutton,FALSE,FALSE,10);
    gtk_signal_connect(GTK_OBJECT(haltbutton),"toggled",
		       GTK_SIGNAL_FUNC(cancel_toggle),(gpointer)NULL);
    gtk_widget_install_accelerator(GTK_WIDGET(haltbutton),main_accel,
				   "clicked",'X',0);

    gtk_widget_show(haltbutton);

    /* put the box into the main window. */
    gtk_container_add (GTK_CONTAINER (window), box1);
    gtk_widget_show(statusbox);

    gtk_box_pack_start(GTK_BOX(box1), mainbar, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(box1), toolbar, FALSE, FALSE, 3);
    gtk_box_pack_start(GTK_BOX(box1), picwin, TRUE,TRUE,0);
    gtk_box_pack_start(GTK_BOX(box1), statusbox, FALSE, FALSE, 0);
    
    gtk_widget_show(toolbar);
     
    gtk_widget_show(box1);

    gtk_widget_show (window);
    gtk_sync();

};


/*
  Make the thumbnail buttons
 */

void make_frames(){
  int i;
  char camname[20];
  GtkWidget *temp;
  struct picent *pe;

  GdkImlibImage *im,*ims;
  GdkPixmap *p,*m;

  m=NULL;
  /* Allocate the array of pix */
  pics=malloc(sizeof(struct picent)*numpix);

  im=gdk_imlib_create_image_from_xpm_data(blank_xpm);
  ims=gdk_imlib_clone_scaled_image(im,thecam_info->thumb_x*thumbscale,
				   thecam_info->thumb_y*thumbscale);
  gdk_imlib_render(ims,ims->rgb_width,ims->rgb_height);
  p=gdk_imlib_move_image(ims);

  if (pictable!=NULL) gtk_widget_destroy(pictable);
    /* a table will hold all of the 'nails */
  pictable=gtk_table_new(1,cols,TRUE);
  gtk_container_add(GTK_CONTAINER(align),pictable);
  gtk_widget_show(pictable);

  for (i=1;i<=numpix;i++){
    pe=pics+i-1;

    image_name(i,camname);
    if (pe->name==NULL) free(pe->name);
    pe->name=strdup(camname);

    pe->picnum=i;
    pe->status=0;
    camname[8]=0;

    /* A toggle button for each thumbnail */
    pe->mainbox=gtk_vbox_new(FALSE,0);
    gtk_widget_show(pe->mainbox);
    gtk_table_attach_defaults(GTK_TABLE(pictable),pe->mainbox,(i-1)%cols,
			      ((i-1)%cols)+1,(i-1)/cols,((i-1)/cols)+1);
    pe->frame=gtk_toggle_button_new(/*camname+3*/);
    gtk_box_pack_start(GTK_BOX(pe->mainbox), pe->frame, FALSE, FALSE, 0);

    /* Box to pack thumb info */
    pe->box=gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(pe->frame),pe->box);
    gtk_container_border_width (GTK_CONTAINER (pe->mainbox), 3);

    /* Show the camera image "number" */
    pe->number=gtk_label_new(camname+3);
    gtk_box_pack_start(GTK_BOX(pe->box), pe->number, FALSE, FALSE, 0);
    gtk_widget_show(pe->number);
    

    /* Default image */
    pe->preview=gtk_pixmap_new(p,NULL);
    gtk_widget_show(pe->preview);
    gtk_box_pack_start(GTK_BOX(pe->box), pe->preview, FALSE, FALSE, 0);
    gtk_widget_show(pe->box);

    gdk_flush();

    /* Hook in the mouse clicks */
    gtk_signal_connect(GTK_OBJECT(pe->frame),"button_press_event",
		       GTK_SIGNAL_FUNC(thumbclick),(gpointer)pe);

    /* Log cursor entry into the button */
    gtk_signal_connect(GTK_OBJECT(pe->frame),"enter_notify_event",
		       GTK_SIGNAL_FUNC(thumbenter),(gpointer)pe);
    gtk_signal_connect(GTK_OBJECT(pe->frame),"focus_in_event",
		       GTK_SIGNAL_FUNC(thumbenter),(gpointer)pe);

    /* Grab button toggle too */
    gtk_signal_connect(GTK_OBJECT(pe->frame),"toggled",
		       GTK_SIGNAL_FUNC(thumb_toggle),(gpointer)pe);

    pe->namebox=gtk_entry_new();
    gtk_widget_set_name(pe->namebox,"filetext");
    gtk_entry_set_text(GTK_ENTRY(pe->namebox),pe->name);
    gtk_widget_set_usize(pe->namebox,80*thumbscale,0);
    gtk_box_pack_start(GTK_BOX(pe->mainbox), pe->namebox, FALSE, FALSE, 0);
    gtk_widget_show(pe->namebox);
    gtk_signal_connect(GTK_OBJECT(pe->namebox),"enter_notify_event",
		       GTK_SIGNAL_FUNC(thumbenter),(gpointer)pe);
    gtk_signal_connect(GTK_OBJECT(pe->namebox),"focus_in_event",
		       GTK_SIGNAL_FUNC(thumbenter),(gpointer)pe);
    gtk_signal_connect(GTK_OBJECT(pe->namebox),"changed",
		       GTK_SIGNAL_FUNC(fname_changed),(gpointer)pe);

    
    gtk_widget_show(pe->frame);
  };

  gdk_imlib_free_pixmap(p);
  gdk_imlib_kill_image(im);
  gdk_imlib_kill_image(ims);

};
